package controllers

import (
	"encoding/json"
	"errors"

	oclib "cloud.o-forge.io/core/oc-lib"
	"cloud.o-forge.io/core/oc-lib/tools"
	beego "github.com/beego/beego/v2/server/web"
)

// Operations about workflow
type WorkflowController struct {
	beego.Controller
}

var paths = map[tools.DataType]map[tools.METHOD]string{ // paths to call other OC services
	tools.BOOKING: {
		tools.POST:   "/oc/booking/",
		tools.DELETE: "/oc/booking/:id",
	},
	tools.PEER: {
		tools.POST: "/oc/peer",
	},
	tools.WORKFLOW: {
		tools.PUT:    "/oc/workflow/:id?is_remote=true",
		tools.DELETE: "/oc/workflow/:id?is_remote=true",
	},
}

// @Title Search
// @Description search workspace
// @Param	search		path 	string	true		"the word search you want to get"
// @Success 200 {workspace} models.workspace
// @router /search/:search [get]
func (o *WorkflowController) Search() {
	// store and return Id or post with UUID
	search := o.Ctx.Input.Param(":search")
	o.Data["json"] = oclib.Search(nil, search, oclib.LibDataEnum(oclib.WORKFLOW))
	o.ServeJSON()
}

// @Title Update
// @Description create workflows
// @Param	id		path 	string	true		"the workflowid you want to get"
// @Param	body		body 	models.workflow	true		"The workflow content"
// @Success 200 {object} models.workflow
// @router /:id [put]
func (o *WorkflowController) Put() {
	// store and return Id or post with UUID
	var res map[string]interface{}
	id := o.Ctx.Input.Param(":id")
	json.Unmarshal(o.Ctx.Input.CopyBody(10000000), &res)
	caller := tools.NewHTTPCaller(paths) // create a new HTTP caller
	caller.Disabled = oclib.IsQueryParamsEquals(o.Ctx.Input, "is_remote", true)
	o.Ctx.Input.Param("is_remote")
	data := oclib.UpdateOne(oclib.LibDataEnum(oclib.WORKFLOW), res, id, caller)
	o.Data["json"] = data
	o.ServeJSON()
}

// @Title Create
// @Description create workflows
// @Param	data		body 	json	true		"body for data content (Json format)"
// @Success 200 {object} models.workflow
// @router / [post]
func (o *WorkflowController) Post() {

	var res map[string]interface{}
	json.Unmarshal(o.Ctx.Input.CopyBody(10000000), &res)
	caller := tools.NewHTTPCaller(paths) // create a new HTTP caller
	caller.Disabled = oclib.IsQueryParamsEquals(o.Ctx.Input, "is_remote", true)
	data := oclib.StoreOne(oclib.LibDataEnum(oclib.WORKFLOW), res, caller)
	o.Data["json"] = data
	o.ServeJSON()
}

// @Title Publish
// @Description create workflows
// @Param	id		path 	string	true		"the workflowid you want to get"
// @Success 200 {object} models.workflow
// @router /publish/:id [post]
func (o *WorkflowController) Publish() {
	id := o.Ctx.Input.Param(":id")
	data := oclib.LoadOne(oclib.LibDataEnum(oclib.WORKFLOW), id)
	if data.Data != nil { // copy existing workflow as a resource
		o.Data["json"] = oclib.CopyOne(oclib.LibDataEnum(oclib.WORKFLOW_RESOURCE), data.Data.Serialize())
	} else {
		o.Data["json"] = data
	}
	o.ServeJSON()
}

// @Title GetAll
// @Description find workflow by workflowid
// @Success 200 {workflow} models.workflow
// @router / [get]
func (o *WorkflowController) GetAll() {
	o.Data["json"] = oclib.LoadAll(oclib.LibDataEnum(oclib.WORKFLOW))
	o.ServeJSON()
}

// @Title Get
// @Description find workflow by workflowid
// @Param	id		path 	string	true		"the workflowid you want to get"
// @Success 200 {workflow} models.workflow
// @router /:id [get]
func (o *WorkflowController) Get() {
	id := o.Ctx.Input.Param(":id")
	o.Data["json"] = oclib.LoadOne(oclib.LibDataEnum(oclib.WORKFLOW), id)
	o.ServeJSON()
}

// @Title Delete
// @Description delete the workflow
// @Param	id		path 	string	true		"The workflowId you want to delete"
// @Success 200 {string} delete success!
// @router /:id [delete]
func (o *WorkflowController) Delete() {
	id := o.Ctx.Input.Param(":id")
	caller := tools.NewHTTPCaller(paths) // create a new HTTP caller
	caller.Disabled = oclib.IsQueryParamsEquals(o.Ctx.Input, "is_remote", true)
	o.Data["json"] = oclib.DeleteOne(oclib.LibDataEnum(oclib.WORKFLOW), id, caller)
	o.ServeJSON()
}

// @Title Check
// @Description check booking
// @Param	id				path 	string		"the booking workflow id"
// @Param	start_date		path 	string		"the booking start date format "2006-01-02T15:04:05"
// @Param	end_date		path 	string		"the booking end date" format 2006-01-02T15:04:05"
// @Success 200 {object} models.object
// @router /check/:id/:start_date/:end_date [get]
func (o *WorkflowController) Check() {
	// store and return Id or post with UUID
	/*
	 * Check if the booking is available
	 */
	id := o.Ctx.Input.Param(":id")
	if id == "" {
		o.Data["json"] = map[string]interface{}{
			"data": map[string]interface{}{
				"is_available": false,
			},
			"code":  400,
			"error": errors.New("invalid date format"),
		}
	} else {
		res := oclib.LoadOne(oclib.LibDataEnum(oclib.WORKFLOW), id)
		if res.Code == 200 {
			workflow := res.ToWorkflow()
			caller := tools.NewHTTPCaller(map[tools.DataType]map[tools.METHOD]string{ // paths to call other OC services
				tools.PEER: {
					tools.POST: "/oc/peer",
				},
				tools.BOOKING: {
					tools.GET: "/oc/booking/check/:id/" + o.Ctx.Input.Param(":start_date") + "/" + o.Ctx.Input.Param(":end_date"),
				},
			})
			isAvailable, err := workflow.CheckBooking(caller) // check booking
			code := 200
			if !isAvailable { // if not available then its a conflict
				code = 409
			}
			o.Data["json"] = map[string]interface{}{
				"data": map[string]interface{}{
					"is_available": isAvailable,
				},
				"code":  code,
				"error": err,
			}
		} else {
			o.Data["json"] = res
		}

	}
	o.ServeJSON()
}
